/*
 * SplashScreen.java - Splash screen
 *
 * Created on Sep 12, 2002, 10:12 AM
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package JRunMC.util;

import java.awt.*;
// import java.net.URL;
// import java.util.ResourceBundle;

/**
 * This file only uses AWT APIs so that it is displayed faster on startup.
 */
public class SplashScreen extends Canvas {
 //   static ResourceBundle res = ResourceFactory.getBundle();

    public static final long serialVersionUID = 119;

// private members
    private FontMetrics fm;
    private Window win;
    private Image image;
    private Image offscreenImg;
    private Graphics offscreenGfx;
    private int progress;
    private static final int PROGRESS_HEIGHT = 10;
    private static int PROGRESS_LENGTH = 10;
    private static String CurrentMess="";
    private String str;
    private Font font1 = new Font("Dialog", Font.PLAIN, 12);


// - input - message to show
    public SplashScreen( String strMessage) {

        str= strMessage;

        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        setBackground(Color.white);

        setFont(font1);
        fm = getFontMetrics(font1);

      image = getToolkit().getImage(
         getClass().getResource("splash_image/splash.png"));



     if (image == null) {
            System.out.println("Splash image not found");
            System.exit(0);
        }



        //      TODO: move the image tracker to the ImageLoader class
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);

        try {
            tracker.waitForAll();
        } catch (Exception e) {
            System.out.println(this + e.getMessage());
        }

        Frame fr = new Frame();
        fr.setIconImage( image ); 

//        fr.setIconImage(ImageLoader.getAppImageIcon().getImage());
        win = new Window(fr);

        Dimension screen = getToolkit().getScreenSize();
        Dimension size = new Dimension(image.getWidth(this) + 2, image.getHeight(this) + 2 + PROGRESS_HEIGHT);
        win.setSize(size);

        win.setLayout(new BorderLayout());
        win.add(BorderLayout.CENTER, this);

        win.setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);
        win.validate();
        win.setVisible(true); 

        //		synchronized(this)
        //		{
        //			try
        //			{
        //				wait();
        //			}
        //			catch(InterruptedException ie)
        //			{
        //				System.out.println(ie.getMessage());
        //			}
        //		}
    }

    public void setProgressLength(int i) {
        PROGRESS_LENGTH = i;
    }

    public void dispose() {
        win.dispose();
    }

    public synchronized void advance(String mess) {
        CurrentMess=mess;
        progress++;
        repaint();

        // wait for it to be painted to ensure progress is updated
        // continuously
        try {
            wait();
        } catch (InterruptedException ie) {
            System.out.println(this + ie.getMessage());
        }
    }

    public void update(Graphics g) {
        paint(g);
    }

    public synchronized void paint(Graphics g) {
        Dimension size = getSize();

        if (offscreenImg == null) {
            offscreenImg = createImage(size.width, size.height);
            offscreenGfx = offscreenImg.getGraphics();
            offscreenGfx.setFont(getFont());
        }

      //  offscreenGfx.setColor(Color.gray);
        offscreenGfx.setColor( new Color(51, 102, 255) ); 
        offscreenGfx.drawRect(0, 0, size.width - 1, size.height - 1);

        offscreenGfx.drawImage(image, 1, 1, this);

        // XXX: This should not be hardcoded
       //  offscreenGfx.setColor(new Color(168, 173, 189));
        offscreenGfx.setColor(new Color( 153, 255, 255  ));
     offscreenGfx.fillRect(1, image.getHeight(this) + 1, ((win.getWidth() - 2) * progress) / PROGRESS_LENGTH,
            PROGRESS_HEIGHT);

        offscreenGfx.setColor(new Color(51, 102, 255) );
        offscreenGfx.drawString(str, (getWidth() - fm.stringWidth(str)) / 2,
            (image.getHeight(this) + 6) - fm.getDescent() - 20);


         offscreenGfx.setColor(Color.gray);
         offscreenGfx.drawString(CurrentMess, (getWidth() - fm.stringWidth(str)) / 2,
            (image.getHeight(this) + 6) - fm.getDescent() - 5);


        g.drawImage(offscreenImg, 0, 0, this);

        notify();
    }

}
