#!/bin/bash

CURRENT_DIR=`pwd`

echo "installing RunMC:"

#

# Change to address of your CLHEP headers
CLHEPINC="/home/sergei/cern/clhep/include"
# CLHEPINC="/opt/products/CLHEP/1.8.2.0/include"
#
#
# include path to CLHEP library here
CLHEP_DIR="/home/sergei/cern/clhep/lib"
# CLHEP_DIR="/opt/products/CLHEP/1.8.2.0/lib"
#
# specify name of CLHEP libarry
# do this without "lib" in front and the extention ".a"
# NAME_CLHEP="CLHEP-g++.1.8.2.0"
NAME_CLHEP="CLHEP-g++.1.8.1.0"

# define here some additional library (if you need it for some reasons)
MYLIB=
#
# define additional include files
MYINCLUDE=

# specify compilers here (do not change this part!) 
CXX="g++"
CC="gcc"
# set for new compilers (gcc4.0 and above)
# check the version as gcc --version
# FORTCOM="gfortran" - does not work yet! 
# set it for gcc3.4 
FORTCOM="gfortran"
# FORTCOM="gfortran"
AR="ar"
# GNU make 
XMAKE="make"


######################################################################
# do not modify this part !!!
# only for experts
############################### do checks #############################


## test enviromental variable
FCHECK=$RUNMC
if [ -z $FCHECK ]
then
  echo "Error: \$RUNMC is not set!  Stop installation"
  exit 1;
fi


echo " "
echo " "
echo "                --- Compilation of  RunMC Toolkit ---"
echo "            --  It also installs RunMC for a single user -- "
echo " "
echo " "

echo " * ok, \$RUNMC set to $FCHECK";



# is this linux
OSYSTEM=`uname`
if [ "$OSYSTEM" != "Linux" ]
then
echo " Error: operating system=$OSYSTEM, this is not Linux! Stop";
exit 1;
fi


fcheck=`type -p $CC`
if test -x "$fcheck"; 
then
    echo " * ok,  $fcheck -> $CC found" 
else
    echo "Error: $CC does not exists"
    exit 0;
fi


fcheck=`type -p $CXX`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $CXX found"
else
    echo "Error: $CXX does not exists"
    exit 0;
fi


fcheck=`type -p $FORTCOM`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $FORTCOM found"
else
    FORTCOM="g77"
    fcheck=`type -p $FORTCOM`
    if test -x "$fcheck";
    then
      echo " * ok, $fcheck -> $FORTCOM found"
    else
      echo "Error: $FORTCOM does not found"
      exit 0;
    fi
fi

# check make
fcheck=`type -p $XMAKE`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> GNU $XMAKE found"
else
    echo "Error: $XMAKE does not exists"
    exit 0;
fi


fcheck=`type -p $AR`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $AR  found"
else
    echo "Error: $AR does not exists"
    exit 0;
fi


fcheck=`type -p ranlib`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> ranlib found"
else
    echo "Error: ranlib does not exists"
    exit 0;
fi


fcheck=`type -p zip`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> zip found"
else
    echo "Error: zip/unzip does not exists"
    exit 0;
fi

fcheck=`type -p sed`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> sed found"
else
    echo "Error: sed does not exists"
    exit 0;
fi

# fcheck=`type -p lhapdf-config`
# if test -x "$fcheck";
# PDFLIB=""
#then
#    echo " * ok, $fcheck -> lhapdf-config found"
#    PDFLIB=`lhapdf-config --prefix`
#    echo "   - LHAPDFlib is installed in $PDFLIB" 
#    PDFLIBSET=`lhapdf-config --pdfsets-path`
#    echo "   - LHAPDF files are in $PDFLIBSET"
# else
#    echo "Error: lhapdf-config does not exists. The LHAPDF library is not installed!"
#    echo "Download it from http://hepforge.cedar.ac.uk/lhapdf/"
#    exit 0;
# fi


fcheck="/usr/lib/libtermcap.a"
if test -e "$fcheck";
then
    echo " * ok, $fcheck -> libtermcap.a found"
else
    echo "Error: Check your linux or cygwin distro!" 
    echo "Error: To compile CASCADE, you need to install \"libtermcap.a\""
    echo "Error: RunMC expects this library in the standard place: "
    echo "$fcheck" 
    echo "(but it can be at some other .. just do: locate libtermcap.a)"
    exit 0;
fi

incTROOT=$ROOTSYS"/include/TROOT.h"  
libCLHEPLIB=$CLHEP_DIR/"lib"$NAME_CLHEP".a"
incCLHEPLIB=$CLHEPINC/"CLHEP/config/CLHEP.h" 
incCLHEPvector=$CLHEPINC"/CLHEP/Vector/LorentzVector.h"
makeCLHEPLIB="-l"$NAME_CLHEP


# fast check for root existence
fcheck=`type -p root-config`
if test -x "$fcheck";
then
    ROOTCONF=`root-config --version`
    echo " * ok, $fcheck -> ROOT exists"
    echo "   - ROOT $ROOTCONF version found"
    echo "   - Location: $ROOTSYS"
else
    echo " Error: root-config does not exists. ROOT is not installed?"
    echo " Download it from http://root.cern.ch"
    exit 0;
fi

# fast check for root existence 
if [ ! -e $incTROOT ]
then
echo "cannot find ROOT include files !  Is ROOT installed? Stop installation"; exit 1;
else
echo "   - Include files: $ROOTSYS/include/ found";
fi

## fast check for root existence
if [ ! -f $libCLHEPLIB ]
then
echo "cannot find $libCLHEPLIB ! Is CLHEPLIB installed? Stop installation"; exit 1;
else
echo " * ok, $libCLHEPLIB found"
fi



## fast check for CLHEP include existence
if [ ! -f $incCLHEPLIB ]
then
echo "cannot find $incCLHEPLIB ! -> Are CLHEPLIB include files installed? Stop installation"; exit 1;
else
echo " * ok, $incCLHEPLIB found"
fi

## fast check for CLHEP include existence
if [ ! -f $incCLHEPvector ]
then
echo "cannot find $incCLHEPvector -> Are CLHEPLIB include files installed? Stop installation"; exit 1;
else
echo " * ok, $incCLHEPvector  found"
fi



installog=$CURRENT_DIR"/install.log" 
if [ -f $installog ]
then
rm -f $installog
fi

if [ -f "Makefile.in" ]
then
rm -f "Makefile.in"
fi


echo " "


### for checking status
cp -f  $RUNMC"/build/auxil/Makefile" "$RUNMC/build/auxil/check.pipe" 

############################################################
############## making include Makefile ######################
cat > $CURRENT_DIR/Makefile.in <<!
#file generated by install.sh script (S.Chekanov) 

# Change to address of your CLHEP headers
CLHEPINC     =  $CLHEPINC
#
#
# Change to address of your CLHEP library
CLHEP_DIR     = $CLHEP_DIR
#
#
# define here some additional library (if you need it for some reasons)
MYLIB         =
#
# define additional include files
MYINCLUDE     =
#
#
SRC_HOME = \$(shell pwd)

# Compilers
CXX          = $CXX 
FORTCOM      = $FORTCOM 
OBJDIR       = ./
CC           = $CC
DEBUGCXX     = -Wall -Wno-format -fPIC -O2
DEBUG_FORTCOM  = -O
 
# Root variables
# Root variables
ROOTCFLAGS    = \$(shell root-config --cflags)
# lNew was added to handel memory mapped histograms
ROOTLIBS      = \$(shell root-config --libs) -lNew
ROOTGTTLIBS   = \$(shell root-config --glibs)
ROOTLIBS     += -lMinuit
# Assign or add variables
CERNLIB       = -L\$(RUNMC)/main/lib -lCERNLIBlite -lLHAPDF
CERN          = \$(CERNLIB) -lm
CLHEP         = -L\$(CLHEP_DIR) $makeCLHEPLIB
CXXFLAGS     += \$(ROOTCFLAGS)
LIBS         += \$(ROOTLIBS) \$(CERN) \$(CLHEP) \$(MYLIB)
MYINC         =../inc
ALL_INCLUDES =  \$(ROOTCFLAGS) -Iinc -I\$(MYINC) -I\$(CLHEPINC) -I\$(MYINCLUDE)
FORTCOM_INCLUDE  = -Iinc -I\$(CLHEPINC) -I\$(MYINC)

# end Makefile.in
!
#
date 
echo "Makefile.in is done !" 
echo "Create \"install.log\" file with installation details"
echo "Wait! -> installation.. It takes 3-10 minutes depending on CPU"
sleep 1;
auxil/checkstatus.sh &
pidc=$!;

# start installation
cp -f auxil/Makefile Makefile
make -f Makefile > $installog 2>&1
# pid=$!;  wait $pid;


# get errors
MYERRORS=`grep -i "error " $installog`;
if [ -n "$MYERRORS" ]
then
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "There were errors in compilation of the Monte Carlo libraries"
  echo "check $installog" 
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "This could be uncorrectly installed ROOT or CLHEP library"
  echo "Check ROOT  as: cd $RUNMC/build/test; ./check_root.sh"
  echo "Check CLHEP as: cd $RUNMC/build/test; ./check_clhep.sh"
  echo "if by running these scripts you will see errors - reinstall these libraries"
  exit 1;
else
  echo "checking install.log  ->       looks ok";
fi
########
########
echo " "
echo "Notes:"
echo "======"
echo "To run JRunMC (java-based GUI), java 1.5 (or above) should be installed"
echo "Check this by typing: java -version"
echo ""
echo "Installation log file is \"install.log\" "
echo ""
echo "Available commands:"
echo "make clean     - remove all object files"
echo "make cleanmc   - remove all object files for Monte Carlo models"
echo "make cleanall  - clean all above + binary files"
echo "make all       - compile (output on the screen, no any configuration checks" 
echo "install        - compile everything with checks. Output goes to install.log file"
echo "" 
echo "RunMC is installed: binary files in $RUNMC/bin"
echo "                    user project files in $RUNMC/proj"
echo "Note: This is a single-user installation!"
echo "      To install RunMC for all users, run install_global.sh and"
echo "      then  run install_user.sh for each user"


# stop checking
# no pipe
# kill -9 $pidc > /dev/null;  
## remove pipes  
if [ -f $RUNMC"/build/auxil/check.pipe" ]
then
rm -f $RUNMC"/build/auxil/check.pipe"
fi
if [ -f $RUNMC"/build/auxil/install.tmp" ]
then
rm -f $RUNMC"/build/auxil/install.tmp" 
fi
##############

###############

exit 0
