C
C
      SUBROUTINE GGBEAM(NEVENT,EE,YPSI,SIGX,SIGY,SIGZ,AEB)
C**********************************************************************
C
C     interface to call PHOJET (variable energy run) for
C     gamma-gamma collisions via beamstrahlung
C
C     input:     EE      LAB system energy of electron/positron
C                YPSI    beamstrahlung parameter
C                SIGX,Y  transverse bunch dimensions
C                SIGZ    longitudinal bunch dimension
C                AEB     number of electrons/positrons in a bunch
C                NEVENT  number of events to generate
C            from COMMON /LEPCUT/:
C                YMIN1   lower limit of Y
C                        (energy fraction taken by photon from electron)
C                YMAX1   upper cutoff for Y, necessary to avoid
C                        underflows
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( ZERO = 0.D0,
     &            ONE  = 1.D0,
     &            OHALF= 0.5D0,
     &            TWO  = 2.D0,
     &            THREE= 3.D0,
     &            DEPS = 1.D-20,
     &            PI   = 3.14159265359D0 )
C
C  lepton/hadron-photon kinematics
      COMMON /PHOSRC/ PINI(5,2),PFIN(5,2),PGAM(5,2),
     &                GYY(2),GQ2(2),GGECM,GAIMP(2),PFTHE(2),PFPHI(2),
     &                IDPSRC(2),IDBSRC(2),RADSRC(2),AMSRC(2),GAMSRC(2)
      COMMON /LEPCUT/ ECMIN,ECMAX,EEMIN1,EEMIN2,
     &                YMIN1,YMAX1,YMIN2,YMAX2,
     &                Q2MIN1,Q2MAX1,Q2MIN2,Q2MAX2,
     &                THMIN1,THMAX1,THMIN2,THMAX2
C
      PARAMETER (MAXTAB=100)
      DIMENSION P1(4),P2(4),TABCU(0:MAXTAB),TABYL(0:MAXTAB)
      CHARACTER*72 TITLE
C
      WRITE(6,'(//1X,A,I10)') 'GGBEAM:DEBUG:EVENTS TO PROCESS',NEVENT
C  electron data
      RE = 2.818D-12
      ELEM = 0.512D-03
      IDPSRC(1) = 0
      IDBSRC(1) = 0
      IDPSRC(2) = 0
      IDBSRC(2) = 0
C  table of flux function, log interpolation
      IF(YPSI.LE.0.D0) THEN
        YPSI  = 5.D0*RE**2*EE*AEB*137.D0/(6.D0*SIGZ*(SIGX+SIGY)*ELEM)
      ENDIF
      WRITE(6,'(/1X,A,E12.4)') 'GGBEAM:BEAMSTRAHLUNG PARAMETER:',YPSI
      WRITE(6,'(/1X,A,2E12.4)') 'GGBEAM:SIGMA-Z,NE-BUNCH:',SIGZ,AEB
      TT    = 2.D0/3.D0
      OT    = 1.D0/3.D0
C     GAOT  = DGAMMA(OT)
      GAOT  = 2.6789385347D0
      AKAP  = TT/YPSI
      WW    = ONE/(6.D0*SQRT(AKAP))
      ANGAM = 5.D0*SIGZ*ELEM/(137.D0**2*TWO*RE*EE)
     &       *YPSI/SQRT(ONE+YPSI**TT)
C
      YMIN = YMIN1
      YMAX = MIN(YMAX1,0.9D0)
      TABCU(0) = ZERO
      TABYL(0) = LOG(YMIN)
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      FLUX = 0.D0
*     WRITE(6,'(1X,A,I5)') 'GGBEAM:DEBUG:TABLE OF PHOTON FLUX',MAXTAB
      DO 100 I=1,MAXTAB
        Y = EXP(TABYL(0)+DELLY*DBLE(I-1))
        GG = ONE-OHALF*(1-Y)**TT*(ONE-Y+(ONE+Y)*SQRT(ONE+YPSI**TT))
        FF = AKAP**OT/GAOT/Y**TT/(ONE-Y)**OT*EXP(-AKAP*Y/(ONE-Y))
     &      *((ONE-WW)/GG*(ONE-(ONE-EXP(-ANGAM*GG))/(ANGAM*GG))
     &      +WW*(ONE-(ONE-EXP(-ANGAM))/ANGAM))
        TABCU(I) = TABCU(I-1)+FF*Y
        TABYL(I) = LOG(Y)
        FLUX = FLUX+Y*FF
*       WRITE(6,'(5X,2E15.4)') Y,FF
 100  CONTINUE
      FLUX = FLUX*DELLY
      WRITE(6,'(1X,A,E12.4)') 
     &  'GGBEAM:DEBUG:INTEGRATED FLUX (ONE SIDE):',FLUX
C
      EE1 = EE
      EE2 = EE
C  photon 1
      EGAM = YMAX*EE
      P1(1) = ZERO
      P1(2) = ZERO
      P1(3) = EGAM
      P1(4) = EGAM
C  photon 2
      EGAM = YMAX*EE
      P2(1) = ZERO
      P2(2) = ZERO
      P2(3) = -EGAM
      P2(4) = EGAM
      CALL SETPAR(1,22,0,ZERO)
      CALL SETPAR(2,22,0,ZERO)
      CALL EVENT(-1,P1,P2,SIGMAX,IREJ)
      CALL POHIST(-1,WEIGHT)
      CALL POLUHI(-1,WEIGHT)
C
C  generation of events
      ECUP = 2.D0*SQRT(YMAX*YMAX*EE1*EE2)*1.1D0
      ECUPL = LOG(ECUP)
      ECMINL = LOG(ECMIN)
      CALL NEWHIS(ECMINL,ECUPL,ZERO,P1,49,IGGLUM)
      CALL NEWHIS(YMIN,YMAX,ZERO,P1,49,IHFLX1)
      CALL NEWHIS(YMIN,YMAX,ZERO,P1,49,IHFLX2)
      CALL NEWHIS(YMIN,YMAX,ZERO,P1,49,IHFLY1)
      CALL NEWHIS(YMIN,YMAX,ZERO,P1,49,IHFLY2)
      AY1  = ZERO
      AY2  = ZERO
      AYS1 = ZERO
      AYS2 = ZERO
      NITER = NEVENT
      ITRY = 0
      ITRW = 0
      DO 200 I=1,NITER
 150    CONTINUE
        ITRY = ITRY+1
 175    CONTINUE
        ITRW = ITRW+1
        XI = RNDM(AY1)*TABCU(MAXTAB)
        DO 110 K=1,MAXTAB
          IF(TABCU(K).GE.XI) THEN
            Y1 = DELLY/(TABCU(K)-TABCU(K-1))*(XI-TABCU(K-1))+TABYL(K-1)
            Y1 = EXP(Y1)
            GOTO 120
          ENDIF
 110    CONTINUE
        Y1 = YMAX
 120    CONTINUE
        XI = RNDM(AY2)*TABCU(MAXTAB)
        DO 130 K=1,MAXTAB
          IF(TABCU(K).GE.XI) THEN
            Y2 = DELLY/(TABCU(K)-TABCU(K-1))*(XI-TABCU(K-1))+TABYL(K-1)
            Y2 = EXP(Y2)
            GOTO 140
          ENDIF
 130    CONTINUE
        Y2 = YMAX
 140    CONTINUE
        CALL FILHIS(Y1,ONE,IHFLX1)
        CALL ADDHIS(IHFLX1)
        CALL FILHIS(Y2,ONE,IHFLX2)
        CALL ADDHIS(IHFLX2)
        Q2P1 = ZERO
        Q2P2 = ZERO
        GYY(1) = Y1
        GQ2(1) = Q2P1
        GYY(2) = Y2
        GQ2(2) = Q2P2
C  incoming electron 1
        PINI(1,1) = ZERO
        PINI(2,1) = ZERO
        PINI(3,1) = EE1
        PINI(4,1) = EE1
        PINI(5,1) = ZERO
C  outgoing electron 1
        YQ2 = SQRT((ONE-Y1)*Q2P2)
        Q2E = Q2P1/(4.D0*EE1)
        E1Y = EE1*(ONE-Y1)
        CALL SFECFE(SIF,COF)
        PFIN(1,1) = YQ2*COF
        PFIN(2,1) = YQ2*SIF
        PFIN(3,1) = E1Y-Q2E
        PFIN(4,1) = E1Y+Q2E
        PFIN(5,1) = ZERO
C  photon 1
        P1(1) = -PFIN(1,1)
        P1(2) = -PFIN(2,1)
        P1(3) = PINI(3,1)-PFIN(3,1)
        P1(4) = PINI(4,1)-PFIN(4,1)
C  incoming electron 2
        PINI(1,2) = ZERO
        PINI(2,2) = ZERO
        PINI(3,2) = -EE2
        PINI(4,2) = EE2
        PINI(5,2) = ZERO
C  outgoing electron 2
        YQ2 = SQRT((ONE-Y2)*Q2P2)
        Q2E = Q2P2/(4.D0*EE2)
        E1Y = EE2*(ONE-Y2)
        CALL SFECFE(SIF,COF)
        PFIN(1,2) = YQ2*COF
        PFIN(2,2) = YQ2*SIF
        PFIN(3,2) = -E1Y+Q2E
        PFIN(4,2) = E1Y+Q2E
        PFIN(5,2) = ZERO
C  photon 2
        P2(1) = -PFIN(1,2)
        P2(2) = -PFIN(2,2)
        P2(3) = PINI(3,2)-PFIN(3,2)
        P2(4) = PINI(4,2)-PFIN(4,2)
C  ECMS cut
        GGECM = (P1(4)+P2(4))**2-(P1(1)+P2(1))**2
     &         -(P1(2)+P2(2))**2-(P1(3)+P2(3))**2
        IF(GGECM.LT.0.1D0) GOTO 175
        GGECM = SQRT(GGECM)
        IF((GGECM.LT.ECMIN).OR.(GGECM.GT.ECMAX)) GOTO 175
C
        PGAM(1,1) = P1(1)
        PGAM(2,1) = P1(2)
        PGAM(3,1) = P1(3)
        PGAM(4,1) = P1(4)
        PGAM(5,1) = ZERO
        PGAM(1,2) = P2(1)
        PGAM(2,2) = P2(2)
        PGAM(3,2) = P2(3)
        PGAM(4,2) = P2(4)
        PGAM(5,2) = ZERO
        CALL PRESEL(5,IREJ)
        IF(IREJ.NE.0) GOTO 175
C  event generation
        CALL EVENT(1,P1,P2,SIGCUR,IREJ)
        IF(IREJ.NE.0) GOTO 150
	GGECML = LOG(GGECM)
        CALL FILHIS(GGECML,ONE,IGGLUM)
        CALL ADDHIS(IGGLUM)
        CALL FILHIS(Y1,ONE,IHFLY1)
        CALL ADDHIS(IHFLY1)
        CALL FILHIS(Y2,ONE,IHFLY2)
        CALL ADDHIS(IHFLY2)
C  statistics
        AY1  = AY1+Y1
        AYS1 = AYS1+Y1*Y1
        AY2  = AY2+Y2
        AYS2 = AYS2+Y2*Y2
C  histograms
        CALL POHIST(1,ONE)
        CALL POLUHI(1,ONE)
 200  CONTINUE
C
      WGY  = FLUX**2*DBLE(ITRY)/DBLE(ITRW)
      AY1  = AY1/DBLE(NITER)
      AYS1 = AYS1/DBLE(NITER)
      DAY1 = SQRT((AYS1-AY1**2)/DBLE(NITER))
      AY2  = AY2/DBLE(NITER)
      AYS2 = AYS2/DBLE(NITER)
      DAY2 = SQRT((AYS2-AY2**2)/DBLE(NITER))
      WEIGHT = WGY*SIGMAX*DBLE(NITER)/DBLE(ITRY)
C  output of statistics, histograms
      WRITE(6,'(//1X,A,/1X,A,1PE12.3,A,/1X,A)')
     &'=========================================================',
     &' *****   simulated cross section: ',WEIGHT,' mb  *****',
     &'========================================================='
      WRITE(6,'(//1X,A,2I10)')
     &  'GGBEAM:SUMMARY:NITER,ITRY',NITER,ITRY
      WRITE(6,'(1X,A,1P2E12.4)') 'EFFECTIVE WEIGHT (FLUX,TOTAL)',
     &  WGY,WEIGHT
      WRITE(6,'(1X,A,2F10.5)') 'GGBEAM:AVERAGE Y1,DY1 ',AY1,DAY1
      WRITE(6,'(1X,A,2F10.5)') 'GGBEAM:AVERAGE Y2,DY2 ',AY2,DAY2
C
      CALL EVENT(-2,P1,P2,WEIGHT,IREJ)
      IF(NITER.GT.1) THEN
        CALL POHIST(-2,WEIGHT)
        CALL POLUHI(-2,WEIGHT)
      ELSE
        WRITE(6,'(1X,A,I4)') 'GGBEAM:NO OUTPUT OF HISTOGRAMS',NITER
      ENDIF
C
      TITLE = ' BEAMSTRAHLUNG PHOTON-PHOTON LUMINOSITY DPHI/D(ECM-GG)'
      FAC =  WGY
      CALL OUTHIS(IGGLUM,3,TITLE,FAC,1)
      TITLE = ' BEAMSTRAHLUNG PHOTON FLUX SIDE 1'
      CALL OUTHIS(IHFLX1,1,TITLE,FLUX,1)
      TITLE = ' BEAMSTRAHLUNG PHOTON FLUX SIDE 2'
      CALL OUTHIS(IHFLX2,1,TITLE,FLUX,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 1 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY1,1,TITLE,FLUX,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 2 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY2,1,TITLE,FLUX,1)
      END
