C
C
      SUBROUTINE MSHELL(PA1,PA2,XM1,XM2,P1,P2,IREJ)
C********************************************************************
C
C    rescaling of momenta of two partons to put both
C                                       on mass shell
C
C    input:       PA1,PA2   input momentum vectors
C                 XM1,2     desired masses of particles afterwards
C                 P1,P2     changed momentum vectors
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           DEPS   =  1.D-20)
C
      DIMENSION PA1(4),PA2(4),P1(4),P2(4)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
C
C  debug output
      IREJ = 0
      IF(IDEB(40).GE.10) THEN
        WRITE(6,'(1X,A)') 'MSHELL:DEBUG:INPUT MOMENTA:'
        WRITE(6,'(5X,4E12.5)') (PA1(K),K=1,4)
        WRITE(6,'(5X,4E12.5)') (PA2(K),K=1,4)
        WRITE(6,'(5X,A,2E12.3)') 'NEW MASSES:',XM1,XM2
      ENDIF
C  Lorentz transformation into system CMS
      PX = PA1(1)+PA2(1)
      PY = PA1(2)+PA2(2)
      PZ = PA1(3)+PA2(3)
      EE = PA1(4)+PA2(4)
      XMS = EE**2-PX**2-PY**2-PZ**2
      IF(XMS.LT.(XM1+XM2)**2) THEN
        IREJ = 1
        IDEV = 5
        IFAIL(37) = IFAIL(37)+1
        IF(IDEB(40).GT.0) THEN
          WRITE(6,'(/1X,A)') 'MSHELL:REJECT:TOO SMALL CHAIN MASS'
          WRITE(6,'(5X,A,3E12.4)') 'SM,MASSES:',
     &      SIGN(SQRT(ABS(XMS)),XMS),XM1,XM2
          WRITE(6,'(5X,A,4E11.4)') 'PX,PY,PZ,EE:',PX,PY,PZ,EE
          IF(IDEB(40).GE.3) GOTO 55
        ENDIF
        RETURN
      ENDIF
      XMS = SQRT(XMS)
      BGX = PX/XMS
      BGY = PY/XMS
      BGZ = PZ/XMS
      GAM = EE/XMS
      CALL ALTRA(GAM,-BGX,-BGY,-BGZ,PA1(1),PA1(2),PA1(3),
     &           PA1(4),PTOT1,P1(1),P1(2),P1(3),P1(4))
C  rotation angles
      PTOT1 = MAX(DEPS,PTOT1)
      COD= P1(3)/PTOT1
      SID= SQRT((ONE-COD)*(ONE+COD))
      COF=ONE
      SIF=ZERO
      IF(PTOT1*SID.GT.DEPS) THEN
        COF=P1(1)/(SID*PTOT1)
        SIF=P1(2)/(SID*PTOT1)
        ANORF=SQRT(COF*COF+SIF*SIF)
        COF=COF/ANORF
        SIF=SIF/ANORF
      ENDIF
C  new CM momentum and energies (for masses XM1,XM2)
      XM12 = XM1**2
      XM22 = XM2**2
      SS   = XMS**2
      PCMP = XLAMB(SS,XM12,XM22)/(2.D0*XMS)
      EE1  = SQRT(XM12+PCMP**2)
      EE2  = XMS-EE1
C  back rotation
      CALL TRANS(ZERO,ZERO,PCMP,COD,SID,COF,SIF,XX,YY,ZZ)
      CALL ALTRA(GAM,BGX,BGY,BGZ,XX,YY,ZZ,EE1,
     &           PTOT1,P1(1),P1(2),P1(3),P1(4))
      CALL ALTRA(GAM,BGX,BGY,BGZ,-XX,-YY,-ZZ,EE2,
     &           PTOT2,P2(1),P2(2),P2(3),P2(4))
C  check consistency
      DEL = XMS*0.0001
      IF(ABS(PX-P1(1)-P2(1)).GT.DEL) THEN
        IDEV = 1
      ELSE IF(ABS(PY-P1(2)-P2(2)).GT.DEL) THEN
        IDEV = 2
      ELSE IF(ABS(PZ-P1(3)-P2(3)).GT.DEL) THEN
        IDEV = 3
      ELSE IF(ABS(EE-P1(4)-P2(4)).GT.DEL) THEN
        IDEV = 4
      ELSE
        IDEV = 0
      ENDIF
 55   CONTINUE
C  debug output
      IF(IDEV.NE.0) THEN
        WRITE(6,'(/1X,A,I3)') 'MSHELL:WARNING:INCONSISTENT TRAFO',IDEV
        WRITE(6,'(1X,A)') 'MSHELL:INPUT MOMENTA:'
        WRITE(6,'(5X,4E12.5)') (PA1(K),K=1,4)
        WRITE(6,'(5X,4E12.5)') (PA2(K),K=1,4)
        WRITE(6,'(5X,A,3E12.3)') 'AVA.MASS,MASSES:',XMS,XM1,XM2
        WRITE(6,'(1X,A)') 'MSHELL:OUTPUT MOMENTA:'
        WRITE(6,'(5X,4E12.5)') (P1(K),K=1,4)
        WRITE(6,'(5X,4E12.5)') (P2(K),K=1,4)
      ELSE IF(IDEB(40).GE.10) THEN
        WRITE(6,'(1X,A)') 'MSHELL:DEBUG:OUTPUT MOMENTA:'
        WRITE(6,'(5X,4E12.5)') (P1(K),K=1,4)
        WRITE(6,'(5X,4E12.5)') (P2(K),K=1,4)
      ENDIF
      END
