C
C
      SUBROUTINE POHINT(IP,ECM,I1,I2,K1,K2,MSPOM)
C**********************************************************************
C
C     interpolate cross sections and weights for hard scattering
C
C     input:  IP     particle combination
C             ECM    CMS energy
C             I1     first subprocess to calculate
C             I2     last subprocess to calculate
C                    <-1  only scales and cutoffs calculated
C             K1     first variable to calculate
C             K2     last variable to calculate
C             MSPOM  cross sections to use for pt distribution
C                    0  reggeon
C                    >0 pomeron
C
C             for K1 < 3 the soft pt distribution is also calculated
C
C     output: interpolated values in XSECT(6,-1:MAXPRO)
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (OHALF  = 0.5D0,
     &           ZERO   = 0.D0,
     &           ONE    = 1.D0,
     &           TWO    = 2.D0,
     &           DEPS   = 1.D-15,
     &           DEPS2  = 2.D-15,
     &           PLARGE = 1.D20 )
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
      COMMON /REGGE/  ALPOM,ALPOMP,GP(2),B0POM(2),
     &                ALREG,ALREGP,GR(2),B0REG(2),
     &                GPPP,GPPR,B0PPP,B0PPR,
     &                VDMFAC(4),VDMQ2F(4),B0HAR,AKFAC
      COMMON /TWOCHA/ PHISUP(2),RMASS1,RMASS2,VAR
      COMMON /XSECPT/ SIGS,DSIGHP,SIGH,FS,FH,BETAS(3),AAS,PTCON
C  hard scales
      COMMON /HAQQAP/ AQQAL,AQQALI,AQQALF,AQQPD,
     &                NQQAL,NQQALI,NQQALF,NQQPD
C
      COMMON /CUTOFF/ PTCUT(4),CUTMU(4),FPS(4),FPH(4),PSOMIN,XSOMIN
      PARAMETER ( MAXPRO = 16 )
      PARAMETER ( MAXTAB = 20 )
      COMMON /HAXSEC/ XSECTA(4,-1:MAXPRO,4,MAXTAB),XSECT(6,-1:MAXPRO),
     &                MXSECT(0:4,-1:MAXPRO,4),ECMSH(4,MAXTAB),ISTTAB
      COMMON /HASCA / PTWANT,AS,AH,ALNS,ALNH,Z1MAX,Z1DIF,Z2MAX,Z2DIF,
     &                PT,ETAC,ETAD,X1,X2,V,U,QQPD,QQAL,
     &                W,W1,AXX,WEIGHT,MSPR,IREJSC
      COMMON /HAEVNT/ PT1,PT2,NHARD,NTRY,IHARD,ITRY,IREJEV
C
      DIMENSION XP(2)
C
C  running cutoff
      IF(IPAMDL(7).EQ.1) THEN
        PTCUT(IP) = PARMDL(35+IP)
     &             +MAX(ZERO,0.12D0*(LOG10(ECM/50.D0))**3)
      ELSE
        PTCUT(IP) = PARMDL(35+IP)
      ENDIF
C  cutoffs
      PT1    = PTCUT(IP)
      PTWANT = PTCUT(IP)
C  scales for hard scattering
      AQQAL  = PARMDL(109+IP)
      AQQALI = PARMDL(113+IP)
      AQQALF = PARMDL(117+IP)
      AQQPD  = PARMDL(121+IP)
      NQQAL  = IPAMDL(64+IP)
      NQQALI = IPAMDL(68+IP)
      NQQALF = IPAMDL(72+IP)
      NQQPD  = IPAMDL(76+IP)
      IF(IDEB(58).GE.15) THEN
        WRITE(6,'(1X,A,4I3,4E10.3)') 'POHINT:DEBUG:SCALES',
     &    NQQAL,NQQALI,NQQALF,NQQPD,AQQAL,AQQALI,AQQALF,AQQPD
      ENDIF
      IF(I2.LT.-1) RETURN
C
C  double-log interpolation
      IF(ECM.LT.2.1D0*PTCUT(IP)) THEN
        DO 50 M=I1,I2
          DO 60 K=K1,K2
            XSECT(K,M) = ZERO
 60       CONTINUE
 50     CONTINUE
      ELSE
        I=1
 310    CONTINUE
          I = I+1
        IF((ECM.GT.ECMSH(IP,I)).AND.(I.LT.ISTTAB)) GOTO 310
        FAC = LOG(ECM/ECMSH(IP,I-1))/LOG(ECMSH(IP,I)/ECMSH(IP,I-1))
        DO 100 M=I1,I2
          DO 200 K=K1,K2
            XX = LOG(XSECTA(K,M,IP,I-1)+DEPS)
     &      +LOG((XSECTA(K,M,IP,I)+DEPS)/(XSECTA(K,M,IP,I-1)+DEPS))*FAC
            XX = EXP(XX)
            IF(XX.LT.DEPS2) XX = ZERO
            IF(K.EQ.2) XX = XX*1.2D0
            XSECT(K,M) = XX
 200      CONTINUE
 100    CONTINUE
      ENDIF
C
      IF((K1.LT.3).AND.(K2.GE.3)) THEN
C  calculate slope of soft pt distribution
        IF(IP.EQ.1) THEN
          SIGS = GR(1)*GR(2)*ECM**(TWO*(ALREG-ONE))
     &          +GP(1)*GP(2)*ECM**(TWO*(ALPOM-ONE))
*         SIGH   = (XSECT(3,15)+XSECT(3,9))
          SIGH   = XSECT(3,9)
     &             /(VDMQ2F(1)+VDMQ2F(2))/(VDMQ2F(3)+VDMQ2F(4))
*         DSIGHP = (XSECT(4,15)+XSECT(4,9))
          DSIGHP = XSECT(4,9)
     &             /(VDMQ2F(1)+VDMQ2F(2))/(VDMQ2F(3)+VDMQ2F(4))
        ELSE IF(IP.EQ.2) THEN
          SIGS = GP(1)*GPPR*ECM**(TWO*(ALREG-ONE))
     &          +GP(1)*GPPP*ECM**(TWO*(ALPOM-ONE))
*         SIGH   = (XSECT(3,15)+XSECT(3,9))/(VDMQ2F(1)+VDMQ2F(2))
          SIGH   = XSECT(3,9)/(VDMQ2F(1)+VDMQ2F(2))
*         DSIGHP = (XSECT(4,15)+XSECT(4,9))/(VDMQ2F(1)+VDMQ2F(2))
          DSIGHP = XSECT(4,9)/(VDMQ2F(1)+VDMQ2F(2))
        ELSE IF(IP.EQ.3) THEN
          SIGS = GP(2)*GPPR*ECM**(TWO*(ALREG-ONE))
     &          +GP(2)*GPPP*ECM**(TWO*(ALPOM-ONE))
*         SIGH   = (XSECT(3,15)+XSECT(3,9))/(VDMQ2F(3)+VDMQ2F(4))
          SIGH   = XSECT(3,9)/(VDMQ2F(3)+VDMQ2F(4))
*         DSIGHP = (XSECT(4,15)+XSECT(4,9))/(VDMQ2F(3)+VDMQ2F(4))
          DSIGHP = XSECT(4,9)/(VDMQ2F(3)+VDMQ2F(4))
        ELSE
          SIGS = GPPR**2*ECM**(TWO*(ALREG-ONE))
     &          +GPPP**2*ECM**(TWO*(ALPOM-ONE))
*         SIGH   = XSECT(3,15)+XSECT(3,9)
          SIGH   = XSECT(3,9)
*         DSIGHP = XSECT(4,15)+XSECT(4,9)
          DSIGHP = XSECT(4,9)
        ENDIF
        FS = FPS(IP)
        FH = FPH(IP)
        SIGS   = MAX(SIGS,ZERO)
        SIGH   = MAX(SIGH,ZERO)
        DSIGHP = MAX(DSIGHP,ZERO)
        CALL SOFTPT(-1,PT1,PT1,XP,IV,PTS)
      ENDIF
C
 300  CONTINUE
C  debug output
      IF(IDEB(58).GE.15) THEN
        WRITE(6,'(1X,A,I10,3I2,2E10.3)')
     &    'POHINT:DEBUG:WEIGHTS EV,IP,K1,K2,ECM,PTC',
     &    KEVENT,IP,K1,K2,ECM,PTCUT(IP)
        DO 162 M=I1,I2
          WRITE(6,'(5X,2I3,4E12.3)')
     &      M,MXSECT(0,M,IP),(XSECT(K,M),K=K1,K2)
 162    CONTINUE
      ENDIF
      END
