C
C
C***********************************************************************
C
C    THIS IS A PACKAGE CONTAINING A RANDOM NUMBER GENERATOR AND
C    SERVICE ROUTINES.
C    THE ALGORITHM IS FROM
C      'TOWARD A UNVERSAL RANDOM NUMBER GENERATOR'
C      G.MARSAGLIA, A.ZAMAN ;
C           FLORIDA STATE UNIV. PREPRINT FSU-SCRI-87-70
C    IMPLEMENTATION BY K. HAHN  DEC. 88,
C    THIS GENERATOR SHOULD NOT DEPEND ON THE HARD WARE ( IF A REAL HAS
C    AT LEAST 24 SIGNIFICANT BITS IN INTERNAL REPRESENTATION ),
C    THE PERIOD IS ABOUT 2**144,
C    TIME FOR ONE CALL AT IBM-XT IS ABOUT 0.7 MILLISECONDS,
C    THE PACKAGE CONTAINS
C      FUNCTION RNDM(X)                     : GENERATOR
C      SUBROUTINE RNDMST(NA1,NA2,NA3,NB4)   : INITIALIZATION
C      SUBROUTINE RNDMIN(U,C,CD,CM,I,J)     : PUT SEED TO GENERATOR
C      SUBROUTINE RNDMOU(U,C,CD,CM,I,J)     : TAKE SEED FROM GENERATOR
C      SUBROUTINE RNDMTE(IO)                : TEST OF GENERATOR
*
C---
*
C    FUNCTION RNDM(DUMMY)
C       GIVES UNIFORMLY DISTRIBUTED RANDOM NUMBERS  IN [0..1)
C       I  - DUMMY VARIABLE, NOT USED
C    SUBROUTINE RNDMST(NA1,NA2,NA3,NB1)
C       INITIALIZES THE GENERATOR, MUST BE CALLED BEFORE USING RNDM
C       NA1,NA2,NA3,NB1  - VALUES FOR INITIALIZING THE GENERATOR
C                          NA? MUST BE IN 1..178 AND NOT ALL 1;
C                          12,34,56  ARE THE STANDARD VALUES
C                          NB1 MUST BE IN 1..168;
C                          78  IS THE STANDARD VALUE
C    SUBROUTINE RNDMIN(U,C,CD,CM,I,J)
C       PUTS SEED TO GENERATOR ( BRINGS GENERATOR IN THE SAME STATUS
C       AS AFTER THE LAST RNDMOU CALL )
C       U(97),C,CD,CM,I,J  - SEED VALUES AS TAKEN FROM RNDMOU
C    SUBROUTINE RNDMOU(U,C,CD,CM,I,J)
C       TAKES SEED FROM GENERATOR
C       U(97),C,CD,CM,I,J  - SEED VALUES
C    SUBROUTINE RNDMTE(IO)
C       TEST OF THE GENERATOR
C       IO     - DEFINES OUTPUT
C                  = 0  OUTPUT ONLY IF AN ERROR IS DETECTED
C                  = 1  OUTPUT INDEPENDEND ON AN ERROR
C       RNDMTE USES RNDMIN AND RNDMOU TO BRING GENERATOR TO SAME STATUS
C       AS BEFORE CALL OF RNDMTE
C
C***********************************************************************
      DOUBLE PRECISION FUNCTION RNDM(DUMMY)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      COMMON /PORAND/ U(97),C,CD,CM,I,J
      RNDMI = U(I)-U(J)
      IF ( RNDMI.LT.0.D0 ) RNDMI = RNDMI+1.D0
      U(I) = RNDMI
      I    = I-1
      IF ( I.EQ.0 ) I = 97
      J    = J-1
      IF ( J.EQ.0 ) J = 97
      C    = C-CD
      IF ( C.LT.0.D0 ) C = C+CM
      RNDMI = RNDMI-C
      IF ( RNDMI.LT.0.D0 ) RNDMI = RNDMI+1.D0
      RNDM = RNDMI
      END
