*CMZ :  2.03/00 11/06/2002  17.29.42  by  Guenter Grindhammer
*CMZ :  2.00/01 15/01/96  09.49.22  by  Guenter Grindhammer
*CMZ :  1.01/03 16/05/95  19.59.11  by  Guenter Grindhammer
*CMZ :  1.01/00 24/02/95  18.06.42  by  Guenter Grindhammer
*CMZ :          19/01/95  14.33.54  by  David Cussans
*CMZ :          18/01/95  16.50.03  by  David Cussans
*CMZU:  1.00/01 11/09/92  14.13.00  by  Guenter Grindhammer
*CMZ :  1.00/00 25/08/92  11.07.01  by  Guenter Grindhammer
*-- Author :    Guenter Grindhammer   14/03/90

      SUBROUTINE GENOUT

*#**********************************************************************
*#
*#    SUBROUTINE GENOUT
*#
*# PURPOSE: create/add monte carlo generated event banks to list E and
*#          RUNDATA (begin and end job record) to list S for output to
*#          tape and/or output to h1 database (h1db).
*#
*# At begin of job:
*#  - HEAD, HEAR, and $RTX (generator comment bank from steering),
*#    GSTX (generator steering bank) and parameter banks GxxR, GxxI,
*#    and GxxA for each generator module -> BLIST S.
*#  - $RTX, HEAR, GxxR, GxxI, and GxxA -> h1db.
*# For first event:
*#  - GHD_ -> h1db.
*# For every event:
*#  - HEAD, HEAR, GHD_, GKI_, GTR_, GVX_ , and GEVC -> BLIST E.
*# At end of job:
*#  - HEAD and GEND -> to BLIST S.
*#  - GEND -> h1db.
*#
*# CALLED BY: all event generators
*#
*# CALLING: GHEAD,GHEAR,BLIST,GPRIA,GHD,GKI,GTR,GVX,GTRGVX,MDB,GEVC,GEND
*#
*# CHANGED BY: Guenter Grindhammer                     DATE: 20/08/92
*# REASON: change calls to h1db.
*#
*# CHANGED BY: Guenter Grindhammer                     DATE: 11/09/92
*# REASON: add HEAR bank to the begin run record (needed by H1SIM/REC).
*#
*# CHANGED BY: Guenter Grindhammer                     DATE: 24/02/95
*# REASON: include modifications to write logfile and disconnect from
*#         h1db after first event.
*#
*# CHANGED BY: Guenter Grindhammer                     DATE: 16/05/95
*# REASON: disable call to GTRGVX for HERWIG.
*#
*# CHANGED BY: Guenter Grindhammer                     DATE: 11/06/02
*# REASON: (re)install new GTRGVX which takes decay vertex information
*#         from the COMMON /HEPEVT/, filled by the respective MC
*#         generator, and fills the GVX_ bank and updates the GTR_ bank
*#         with the pointers from the GTR_ to the GVX_ bank. GTRGVX is
*#         now called for all generators (ISKIP not needed).
*#
*# CHANGED BY:                                         DATE: dd/mm/yy
*# REASON:
*#**********************************************************************

#include "h1util/jobnr/cjobnr.inc" 
#include "h1mcutil/comdef/cbosmdl.inc" 
#include "h1mcutil/comdef/cbcs.inc" 
#include "h1mcutil/comdef/cgpar.inc" 
#include "h1mcutil/comdef/cgvrs.inc" 

      LOGICAL         FIRST1,FIRST2
      LOGICAL         LOGING
      INTEGER         LOGLUN
      INTEGER         LUNLOG
      EXTERNAL        LUNLOG
      CHARACTER*66    C66

      DATA FIRST1 /.TRUE./, FIRST2 /.TRUE./

*  create parameter banks for a generator module for the begin job
*  record and the h1 database
      IF (BEGJOB) THEN
         LOGLUN = LUNLOG()
         LOGING = ( LOGLUN.GT.0 )
         IF (FIRST1) THEN
            CALL GHEAD
            CALL GHEAR
            CALL BLIST(IW,'S+','HEADHEAR$RTXGSTX')
            WRITE(C66,*) 'DBWRITE HEAR DBTABL="=EAR" FMT="I" LOGRUN=',
     &                    NCCRUN,' DTYPE=1'
            INDB = MDB(C66)

*  If the user has asked for a logfile then print out the =RTX, $RTX
*  and GSTX banks, preceeded by the MC run number and the Job number.
*  NB. When using h1db, the random number seed (ISEED) is JOBNR+1.
*  NB. =RTX and $RTX printout should probably be moved to H1UTIL
            IF (LOGING) THEN
               WRITE(LOGLUN,100) NCCRUN,JOBNR
  100          FORMAT(2X,'Monte-Carlo Run: ', I6/
     &                2X,'Monte-Carlo Job: ', i6)
               CALL GGUSEPR(LOGLUN)
               CALL GRTXPR(LOGLUN)
               CALL GGSTXPR(LOGLUN)
            ENDIF
            FIRST1 = .FALSE.
         ENDIF
         CALL GPRIA
      ENDIF

*  for every event create banks HEAD, HEAR, GHD_, GKI_, GTR_, GVX_, and
*  GEVC and for the first event add the GHD_ bank to the h1 database
      IF (REVENT) THEN
         CALL GHEAD
         CALL GHEAR
         CALL GHD
         CALL GKI
         CALL GTR
         CALL GVX
C         ISKIP = 0
C         DO 200 I=1,NNAMES
C         IF (PRGNAM(I) .EQ. 'HERWIG') ISKIP = 1
C  200    CONTINUE
C         IF (ISKIP .EQ. 0) CALL GTRGVX
ccc         CALL GTRGVX
         CALL GEVC
         CALL BLIST(IW,'E+','HEADHEARGHD GKI GTR GVX GEVC')
         IF (FIRST2) THEN
            WRITE(C66,*) 'DBWRITE "GHD " LOGRUN=',NCCRUN,' DTYPE=1'
            INDB = MDB(C66)
            FIRST2 = .FALSE.
            INDB = MDB('DBPAUSE')

*  If the user has asked for a logfile then print out the GHD_ bank.
            IF (LOGING) THEN
               CALL GGHDPR(LOGLUN)
            ENDIF
         ENDIF
      ENDIF

*  create bank GEND for the end job record and the h1 database
      IF (ENDJOB) THEN
         CALL GHEAD
         CALL GEND
         CALL BLIST(IW,'S+','HEADGEND')
         WRITE(C66,*) 'DBWRITE GEND LOGRUN=',NCCRUN,' DTYPE=1'
         INDB = MDB(C66)

*  If the user has asked for a logfile then print out the GEND bank.
         IF (LOGING) THEN
            CALL GGENDPR(LOGLUN)
         ENDIF
      ENDIF

      RETURN
      END
