*CMZ :          25/06/2004  17.12.04  by  H. Jung
*CMZ :  6.21/20 16/12/2002  23.59.01  by  Unknown
*-- Author :
      SUBROUTINE GPYPAR

*#**********************************************************************
*#
*#    SUBROUTINE GPYPAR
*#
*# PURPOSE: copy updated parameters from PYTHIA 6.1 commons into GENOUT
*#          commons. define names of parameter banks.
*#          note: only the parameters which the user according to the
*#                PYTHIA manual may change are copied.
*#
*# INPUT: PYTHIA commons /PYPARS/ and /PYSUBS/, and the additional
*#        commons /PBEAM1/ and /PBEAM2/
*# INPUT: PYTHIA commons /PYSUBS/,/PYPARS/,/PYDAT1/,/PYDAT3/,
*#        and the additional H1 commons /PBEAM1/ and /PBEAM2/
*#        (Note : /PYDAT1/ and /PYDAT3/ were formerly in JETSET as
*#                /LUDAT1/ and /LUDAT3/
*#
*# OUTPUT: GENOUT COMMON /GPRPRM/, /GPIPRM/, /GPAPRM/, /GPLPRM/,
*#         /GBKNAM/ and /GBKVER/
*#
*# CALLED BY: GENFIL
*#
*# CALLING: UCOPY
*#
*# CHANGED BY: B. Andrieu                                 DATE: 25/04/00
*# REASON: Adaptation to PYTHIA 6.1
*#           -> conversion to double precision
*#           -> merging PYTHIA/JETSET : GJEPAR merged with GPYPAR
*#         Replacement of COMMONs by +SEQs.
*#
*#**********************************************************************

C---additional PYTHIA commons (not part of standard PYTHIA)
c#include "pythia62/h1beam.inc"
C---additional PYTHIA commons (not part of standard PYTHIA)
      CHARACTER*8      FRAME, BEAM, TARGET
      COMMON /PBEAM1/  FRAME, BEAM, TARGET
      SAVE /PBEAM1/
      DOUBLE PRECISION  WIN, PBEAM,    PTARGE
      COMMON /PBEAM2/   WIN, PBEAM(3), PTARGE(3)
      SAVE /PBEAM2/

C---PYTHIA commons
*KEEP,pysubs.
      Integer MSEL,MSELPD,MSUB,KFIN
	Double Precision CKIN
      COMMON/PYSUBS/MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
      SAVE /PYSUBS/
*KEEP,pypars.
      Integer MSTP,MSTI
	Double Precision PARP,PARI
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
      SAVE /PYPARS/
*KEEP,pydat1.
      Integer MSTU,MSTJ
	Double Precision PARU,PARJ
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      SAVE /PYDAT1/
*KEEP,pydat3.
      Integer MDCY,MDME,KFDP
	Double Precision BRAT
      COMMON/PYDAT3/MDCY(500,3),MDME(8000,2),BRAT(8000),KFDP(8000,5)
      SAVE /PYDAT3/
*KEND.

C---GENOUT PARAMETER COMMONS---
#include "h1mcutil/comdef/cgpar.inc"

C---define parameter bank names and parameter version number for GENOUT.
C   the parameter version number starts out being identical to the
C   author's generator version number (2 or 3 digits) and is not changed
C   when only code has been updated but the meaning of the previously
C   defined parameters and the parameter array sizes stayed the same.
      BKNAMR = 'GPYR'
      BKNAMI = 'GPYI'
      BKNAMA = 'GPYA'
C   version number must be >= 100
      BKVERS = 610

C---copy the integer parameters
* First PYTHIA parameters
      INPARM(1) = MSEL
      LIPARM = 1
      CALL UCOPY(MSUB,INPARM(LIPARM+1),NSUB)
      LIPARM = LIPARM + NSUB
      CALL UCOPY(KFIN,INPARM(LIPARM+1),NKFIN)
      LIPARM = LIPARM + NKFIN
      CALL UCOPY(MSTP,INPARM(LIPARM+1),NMSTP)
      LIPARM = LIPARM + NMSTP
* Now add JETSET (embedded in PYTHIA) parameters
      CALL UCOPY(MSTU,INPARM(LIPARM+1),NMSTU)
      LIPARM = LIPARM + NMSTU
      CALL UCOPY(MSTJ,INPARM(LIPARM+1),NMSTJ)
      LIPARM = LIPARM + NMSTJ
      CALL UCOPY(MDCY,INPARM(LIPARM+1),NKC*NMDCY2)
      LIPARM = LIPARM + NKC*NMDCY2
      CALL UCOPY(MDME,INPARM(LIPARM+1),NIDC*NMDME2)
      LIPARM = LIPARM + NIDC*NMDME2
      CALL UCOPY(KFDP,INPARM(LIPARM+1),NIDC*NKFDP2)
      LIPARM = LIPARM + NIDC*NKFDP2

C---copy the real parameters
* First PYTHIA parameters
      REPARM(1) = sngl(WIN)
      reparm(2) = sngl(pbeam(1))
      reparm(3) = sngl(pbeam(2))
      reparm(4) = sngl(pbeam(3))
      reparm(5) = sngl(ptarge(1))
      reparm(6) = sngl(ptarge(2))
      reparm(7) = sngl(ptarge(3))
      LRPARM =  7
      do ik=1,NCKIN
         reparm(LRPARM+ik) = sngl(ckin(ik))
      enddo
      LRPARM = LRPARM + NCKIN
      do ik=1,NPARP
         reparm(LRPARM+ik) = sngl(parp(ik))
      enddo
      LRPARM = LRPARM + NPARP
* Now add JETSET (embedded in PYTHIA) parameters
      do ik=1,NPARU
 	reparm(LRPARM+ik) = sngl(paru(ik))
      enddo
      LRPARM = LRPARM + NPARU
      do ik=1,NPARJ
 	reparm(LRPARM+ik) = sngl(parj(ik))
      enddo
      LRPARM = LRPARM + NPARJ
      do ik=1,NIDC
 	reparm(LRPARM+ik) = sngl(brat(ik))
      enddo
      LRPARM = LRPARM + NIDC

C---finally copy the character parameters
* First PYTHIA parameters
      CHPARM(1) = FRAME(1:4)
      CHPARM(2) = FRAME(5:8)
      CHPARM(3) = BEAM(1:4)
      CHPARM(4) = BEAM(5:8)
      CHPARM(5) = TARGET(1:4)
      CHPARM(6) = TARGET(5:8)
      LAPARM =    6

      RETURN
      END
