// * This code is licensed under:
// * JRunMC License, Version 1.0
// * - for license details see http://runmc.sourceforge.net/License.txt
// *
// * Copyright (c) 2005 by S.Chekanov (chekanov@mail.desy.de).
// * All rights reserved.


#include <TROOT.h>
#include <TPad.h>
#include <TMapFile.h>
#include <TH1.h>
#include <TH2.h>
#include <TSystem.h>
#include <RVersion.h>

#include <jni.h>
#include <stdio.h>
#include <math.h>
#include "AIDAEmbed.h"
#include <iostream>

using namespace std;


JNIEXPORT jint JNICALL Java_JRunMC_hlookup_AIDAEmbed_jnipower(
                                                 JNIEnv *env,
                                                 jclass in_cls,
                                                 jint in_BinX,
                                                 jint in_BinY,
                                                 jstring in_string,
                                                 jfloatArray myarray,
                                                 jfloatArray errors)

  {

  jboolean blnIsCopy;

  const char* Hname = env->GetStringUTFChars(in_string, &blnIsCopy);

  float* c_array = env->GetFloatArrayElements (myarray, NULL);
  float* e_array = env->GetFloatArrayElements (errors, NULL);

  int c_bx = in_BinX;
  int c_by = in_BinY;

//   cout << "in_BinX =" << c_bx << endl;
//   cout << "in_BinY =" << c_by << endl;

// get size 
  jsize len = env->GetArrayLength(myarray);

  int ntt=0;

  float sum=0.0;
  for (int i=0; i<len; i++) {
    sum += c_array[i];
  }
//  cout << "Size=" << (int)len  << endl;

  char * RUNDIR; RUNDIR=getenv("RUNMC");
  char MemoryMappedFile[500];
  strcpy(MemoryMappedFile,RUNDIR);
  strncat(MemoryMappedFile,"/pipes/memfile.map",strlen("/pipes/memfile.map") );
  TMapFile* mfile = 0;
  mfile = TMapFile::Create(MemoryMappedFile);
 

// make sure that the reaeze above 5.04
// #if ROOT_VERSION_CODE > 328704 
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,06,0)
// Fix message: *** Break *** write on a pipe with no one to read it 
   char ErrorFile[500];
   strcpy(ErrorFile,RUNDIR);
   strncat(ErrorFile,"/main/tmp/shmem.log",strlen("/main/tmp/shmem.log") );
   gSystem->RedirectOutput(ErrorFile, "a");
#else
    // no this feature implemented 
#endif



  TH1F    *hpx = 0;
  TH2F    *hpx2D = 0;
  TObject *ll =0;


  if (in_BinX>0 && in_BinY ==0 ) {
    
    ll=mfile->Get(Hname, hpx);
    if ( ll == 0) return 0;

    hpx      = (TH1F*)ll;
    ntt=(int)(hpx->GetEntries());
  for (int i=0; i<len; i++) {
     int j = i+1; 
     c_array[i]= hpx->GetBinContent(j);
     e_array[i]= hpx->GetBinError(j);
  }

  // copy back the array
   env->ReleaseFloatArrayElements (myarray, c_array, 0);
   env->ReleaseFloatArrayElements (errors, e_array, 0);
   mfile->Close();
   hpx->Delete();
   return ntt;
  }

// 2D 
  if (in_BinX>0 && in_BinY >0 ) {

    ll=mfile->Get(Hname, hpx2D);
    if ( ll == 0) return 0;

    int nn=0; 
    hpx2D      = (TH2F*)ll;
    ntt=(int)(hpx2D->GetEntries());
//    cout << "2 D entries =" << ntt << endl;

  for (int i=0; i<c_bx; i++) {
       int j = i+1;
         for (int k=0; k<c_by; k++) { 
          int m = k+1;

          c_array[nn]= hpx2D->GetBinContent(j,m);
          e_array[nn]= hpx2D->GetBinError(j,m);
          nn++; 
          }
   } 

    // copy back the array
   env->ReleaseFloatArrayElements (myarray, c_array, 0);
   env->ReleaseFloatArrayElements (errors, e_array, 0);
   mfile->Close();
   hpx2D->Delete();
   return ntt;

 } //  end 2D 


   return -1;
 
}
