// * This code is licensed under:
// * RunMC License, Version 1.0
// * - for license details see http://runmc.sourceforge.net/License.txt
// *
// * Copyright (c) 2005 by S.Chekanov (chekanov@mail.desy.de).
// * All rights reserved.


#include <TROOT.h>
#include <TTree.h>
#include <TChain.h>

#include<iostream>
#include<fstream>
using namespace std;

#include<SetIni.h>    // include class containing initializations for RunMC

static char * NtupleNames[1000];
static Int_t  NTnumber;



#ifdef  PYTHIA57
 #include "hepevt_pythia57.h"
#endif
 
#ifdef  PYTHIA6
  #include "hepevt_pythia62.h"
#endif
 
#include<hepevt_runmc.h>
#include<heplist.h>

static TChain *chain1 = new TChain("trunmc"); 



Int_t  get_rtup( Int_t  mode ) { 

  extern SetIni inival; 
  static Int_t  nbytes = 0;
  hepenvtrunmc rntup; // HEPEVT local for output 
  static Int_t ntotntuple=0;

 
  if (mode == -1) {   

// get ntuples
  ifstream myin(".runmc.tup", ios::in); //only check at the beginning
  if (!myin) {
  cout << "Cannot open .runmc.tup" << endl; return 1;};

  char buffer[300];
  NTnumber=-1;
  while (! myin.eof() )
  {
     NTnumber++;
     myin.getline (buffer,300);
     NtupleNames[NTnumber] = new char[strlen(buffer)+1];
     strcpy(NtupleNames[NTnumber],buffer);
     }
  myin.close();

     cout << "RUNMC: total number of ntuples: " << NTnumber << endl;
     for (int m=0; m<NTnumber; m++) {
     cout << "   ntuple: " << NtupleNames[m] << endl;
     }







  for (Int_t m=0; m<NTnumber; m++) chain1->Add(NtupleNames[m]);
  chain1->SetBranchAddress("NHEP",&rntup.NHEP);
  chain1->SetBranchAddress("IDHEP",&rntup.IDHEP);
  chain1->SetBranchAddress("ISTCHA",&rntup.ISTCHA);
  chain1->SetBranchAddress("JMOHEP",&rntup.JMOHEP);
  chain1->SetBranchAddress("JDAHEP",&rntup.JDAHEP);
  chain1->SetBranchAddress("PX",&rntup.PHEP_X);
  chain1->SetBranchAddress("PY",&rntup.PHEP_Y);
  chain1->SetBranchAddress("PZ",&rntup.PHEP_Z);
  chain1->SetBranchAddress("E",&rntup.PHEP_E);


  ntotntuple= (long int)chain1->GetEntries();
  cout << "RUNMC: total number of events in this ntuple = " << ntotntuple << endl;
// run over this number only
  if (ntotntuple < inival.Nevents ) inival.Nevents=ntotntuple;



  return 0;

  } // end if 



   Int_t i=mode; 
   nbytes += chain1->GetEntry(i);  
  

   extern hepenvtcommon hepevt_ ; // To access the external HEPEVT
//   cout << "Particles = " << rntup.NHEP << endl;

// get isthep 
   Float_t tmp;

   hepevt_.NEVHEP=i;
   hepevt_.NHEP=rntup.NHEP;
        for (Int_t nn=0; nn < hepevt_.NHEP; nn++) {

  tmp=rntup.ISTCHA[nn]/100.;
//  hepevt_.ISTHEP[nn]=TMath::Nint(tmp); 
  hepevt_.ISTHEP[nn]=(int)tmp;
  hepevt_.IDHEP[nn]=rntup.IDHEP[nn];
  hepevt_.JMOHEP[nn][0]=rntup.JMOHEP[nn];
  hepevt_.JMOHEP[nn][1]=0; 
  hepevt_.JDAHEP[nn][0]=rntup.JDAHEP[nn];
  hepevt_.JDAHEP[nn][1]=0; 
  hepevt_.PHEP[nn][0]=rntup.PHEP_X[nn];
  hepevt_.PHEP[nn][1]=rntup.PHEP_Y[nn];
  hepevt_.PHEP[nn][2]=rntup.PHEP_Z[nn];
  hepevt_.PHEP[nn][3]=rntup.PHEP_E[nn];
  hepevt_.PHEP[nn][4]=0.0; 
  hepevt_.VHEP[nn][0]=0.0;
  hepevt_.VHEP[nn][1]=0.0;
  hepevt_.VHEP[nn][2]=0.0;
  hepevt_.VHEP[nn][3]=0.0; }


// ouutput for 3 events:
 
    if (mode < 3) {
    printf("\n\n RUNMC event from RTUP    = %5d \n",mode);
    printf(" Total number of particles  = %5d \n",hepevt_.NHEP);
// loop over all particles !
    for (Int_t  nn=0; nn<hepevt_.NHEP;nn++) {
    printf("%4d ISTHEP=%5d  IDHEP=%5d JMOHEP[0]=%5d JMOHEP[1]=%5d JDAHEP[0]=%5d JDAHEP[1]=%5d \n",
    nn,hepevt_.ISTHEP[nn],hepevt_.IDHEP[nn],hepevt_.JMOHEP[nn][0],hepevt_.JMOHEP[nn][1],hepevt_.JDAHEP[nn][0],hepevt_.JDAHEP[nn][1] );
    printf("     Px=%6.3f  Py=%6.3f  Pz=%6.3f  E=%6.3f Mass=%8.4f\n",
    hepevt_.PHEP[nn][0], hepevt_.PHEP[nn][1], hepevt_.PHEP[nn][2], hepevt_.PHEP[nn][3], hepevt_.PHEP[nn][4]);
    printf("     V1=%6.5f  V2=%6.5f  V3=%6.5f  V4=%6.5f \n",
    hepevt_.VHEP[nn][0], hepevt_.VHEP[nn][1], hepevt_.VHEP[nn][2], hepevt_.VHEP[nn][3]  );
 
    } // end loop over particles
 
   }

 
 return 0;
}
