/*
 * CobolTokenMarker.java - Dawn token marker
 * Copyright (C) 2000 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.syntax;

import javax.swing.text.Segment;

public class KumacTokenMarker extends TokenMarker
{
  // private members
  private static KeywordMap kumacKeywords;
  private KeywordMap keywords;

  private int lastOffset;
  private int lastKeyword;

  public KumacTokenMarker()
  {
    this.keywords = getKeywords();
  }

  public byte markTokensImpl(byte token, Segment line, int lineIndex)
  {
    char[] array = line.array;
    int offset = line.offset;
    lastOffset = offset;
    lastKeyword = offset;
    int length = line.count + offset;

loop: for(int i = offset; i < length; i++)
    {
      int i1 = (i+1);

      char c = array[i];

      switch(token)
      {
        case Token.NULL:
          switch(c)
          {
            case 'E':
              if (SyntaxUtilities.regionMatches(false, line, i1, "XEC SQL"))
              {
                doKeyword(line, i, c);
                i += 7;
                token = Token.KEYWORD3;
              }
              break;
            case '*':
              addToken(i - lastOffset, token);
              addToken(length - i, Token.COMMENT1);
              token = Token.NULL;
              lastOffset = lastKeyword = length;
              break loop;
            case '"':
              doKeyword(line, i, c);
              addToken(i - lastOffset,token);
              token = Token.LITERAL1;
              lastOffset = lastKeyword = i;
              break;
            case '\'':
              doKeyword(line, i, c);
              addToken(i - lastOffset,token);
              token = Token.LITERAL2;
              lastOffset = lastKeyword = i;
              break;
            case ' ':
              doKeyword(line, i, c);
          }
          break;
        case Token.KEYWORD3:
          if (c == 'E')
          {
            if (SyntaxUtilities.regionMatches(false, line, i1, "ND-EXEC"))
            {
              i += 8;
              addToken(i - lastOffset, token);
              token = Token.NULL;
              lastOffset = lastKeyword = i;
            }
          }
          break;
        case Token.LITERAL1:
          if(c == '"')
          {
            addToken(i1 - lastOffset, token);
            token = Token.NULL;
            lastOffset = lastKeyword = i1;
          }
          break;
        case Token.LITERAL2:
          if(c == '\'')
          {
            addToken(i1 - lastOffset, Token.LITERAL1);
            token = Token.NULL;
            lastOffset = lastKeyword = i1;
          }
          break;
        default:
          throw new InternalError("Invalid state: " + token);
      }
    }

    if (token == Token.NULL)
      doKeyword(line, length, '\0');

    switch(token)
    {
//      case Token.LITERAL1:
//      case Token.LITERAL2:
//        addToken(length - lastOffset, Token.INVALID);
//        token = Token.NULL;
//        break;
      //case Token.KEYWORD2:
      //  addToken(length - lastOffset, token);
      //  token = Token.NULL;
      default:
        addToken(length - lastOffset, token);
        break;
    }

    return token;
  }

  public static KeywordMap getKeywords()
  {
    if (kumacKeywords == null)
    {
      kumacKeywords = new KeywordMap(true);
      kumacKeywords.add("macro", Token.KEYWORD3);
      kumacKeywords.add("set", Token.KEYWORD3);
      kumacKeywords.add("title", Token.KEYWORD3);
      kumacKeywords.add("atitle", Token.KEYWORD3);
      kumacKeywords.add("zone", Token.KEYWORD3);
      kumacKeywords.add("opt", Token.KEYWORD3);
      kumacKeywords.add("if", Token.KEYWORD3);
      kumacKeywords.add("end", Token.KEYWORD3);
      kumacKeywords.add("endif", Token.KEYWORD3);
      kumacKeywords.add("nbox", Token.KEYWORD3);
      kumacKeywords.add("asiz", Token.KEYWORD3);
      kumacKeywords.add("utit",Token.KEYWORD3);
      kumacKeywords.add("option", Token.KEYWORD3);
      kumacKeywords.add("xwin", Token.KEYWORD3);
      kumacKeywords.add("ywin", Token.KEYWORD3);
      kumacKeywords.add("AND", Token.KEYWORD3);

      kumacKeywords.add("clr", Token.KEYWORD1);
      kumacKeywords.add("ksiz", Token.KEYWORD1);

      kumacKeywords.add("DATA", Token.KEYWORD1);
      kumacKeywords.add("mess", Token.KEYWORD3);
      kumacKeywords.add("ELSE", Token.KEYWORD3);
      kumacKeywords.add("END", Token.KEYWORD3);
      kumacKeywords.add("print", Token.KEYWORD1);
      kumacKeywords.add("do", Token.KEYWORD1);
      kumacKeywords.add("FILE", Token.KEYWORD1);
      kumacKeywords.add("FUNCTION", Token.KEYWORD1);

      kumacKeywords.add("IF", Token.KEYWORD1);
      kumacKeywords.add("print", Token.KEYWORD1);

  // === Commands ===
      kumacKeywords.add("CALL", Token.KEYWORD1);
      kumacKeywords.add("CLOSE", Token.KEYWORD1);
      kumacKeywords.add("CONTINUE", Token.KEYWORD1);
      kumacKeywords.add("DO", Token.KEYWORD1);
      kumacKeywords.add("ELSE", Token.KEYWORD1);
      kumacKeywords.add("ELSEIF", Token.KEYWORD1);
      kumacKeywords.add("ENDIF", Token.KEYWORD1);
      kumacKeywords.add("GOTO", Token.KEYWORD1);
      kumacKeywords.add("GO TO", Token.KEYWORD1);
      kumacKeywords.add("IF", Token.KEYWORD1);
      kumacKeywords.add("INDEX", Token.KEYWORD1);
      kumacKeywords.add("INQUIRE", Token.KEYWORD1);
      kumacKeywords.add("OPEN", Token.KEYWORD1);
      kumacKeywords.add("PRINT", Token.KEYWORD1);
      kumacKeywords.add("READ", Token.KEYWORD1);
      kumacKeywords.add("RETURN", Token.KEYWORD1);
      kumacKeywords.add("THEN", Token.KEYWORD1);
      kumacKeywords.add("WRITE", Token.KEYWORD1);

      // === Compiler directives ===
      kumacKeywords.add("BLOCK DATA", Token.KEYWORD2);
      kumacKeywords.add("COMPILER", Token.KEYWORD2);
      kumacKeywords.add("END", Token.KEYWORD2);
      kumacKeywords.add("ENTRY", Token.KEYWORD2);
      kumacKeywords.add("FUNCTION", Token.KEYWORD2);
      kumacKeywords.add("INCLUDE", Token.KEYWORD2);
      kumacKeywords.add("SUBROUTINE", Token.KEYWORD2);

      // === Data types (etc.) ===
      kumacKeywords.add("CHARACTER", Token.KEYWORD3);
      kumacKeywords.add("DATA", Token.KEYWORD3);
      kumacKeywords.add("DEFINE", Token.KEYWORD3);
      kumacKeywords.add("EQUIVALENCE", Token.KEYWORD3);
      kumacKeywords.add("IMPLICIT", Token.KEYWORD3);
      kumacKeywords.add("INTEGER", Token.KEYWORD3);
      kumacKeywords.add("LOGICAL", Token.KEYWORD3);
      kumacKeywords.add("PARAMETER", Token.KEYWORD3);
      kumacKeywords.add("REAL", Token.KEYWORD3);


  // === Operators ===
      kumacKeywords.add(".AND.", Token.OPERATOR);
      kumacKeywords.add(".EQ.", Token.OPERATOR);
      kumacKeywords.add(".NE.", Token.OPERATOR);
      kumacKeywords.add(".NOT.", Token.OPERATOR);
      kumacKeywords.add(".OR.", Token.OPERATOR);
      kumacKeywords.add("+", Token.OPERATOR);
      kumacKeywords.add("-", Token.OPERATOR);
      kumacKeywords.add("*", Token.OPERATOR);
      kumacKeywords.add("**", Token.OPERATOR);
      kumacKeywords.add("/", Token.OPERATOR);

      // === Literals ===
      kumacKeywords.add(".FALSE.", Token.LITERAL2);
      kumacKeywords.add(".TRUE.", Token.LITERAL2);



    }

    return kumacKeywords;
  }

  private boolean doKeyword(Segment line, int i, char c)
  {
    int i1 = i + 1;

    int len = i - lastKeyword;
    byte id = keywords.lookup(line, lastKeyword, len);
    if (id != Token.NULL)
    {
      if(lastKeyword != lastOffset)
        addToken(lastKeyword - lastOffset, Token.NULL);
      addToken(len, id);
      lastOffset = i;
    }
    lastKeyword = i1;
    return false;
  }
}

// End of CobolTokenMarker.java
